"""
Environment configuration for Judo Framework examples
"""

from judo.behave import *

# Import all Judo hooks
before_all = before_all_judo
before_feature = before_feature_judo
after_feature = after_feature_judo
before_scenario = before_scenario_judo
after_scenario = after_scenario_judo
before_step = before_step_judo
after_step = after_step_judo
after_all = after_all_judo