"""
Judo Framework - Complete API Testing Framework for Python
Inspired by Karate Framework with full DSL implementation

Created by Felipe Farias at CENTYC - Centro Latinoamericano de Testing y Calidad del Software
https://www.centyc.cl
"""

from .core.judo import Judo
from .core.response import JudoResponse
from .core.matcher import Matcher
from .core.variables import VariableManager
from .http.client import HttpClient
from .mock.server import MockServer
from .reporting.reporter import JudoReporter
from .reporting.html_reporter import HTMLReporter
from .runner.base_runner import BaseRunner
from .runner.test_suite import TestSuite, CommonSuites
from .utils.helpers import *
from .genai.client import GenAIClient
from .genai.judge import GenAIJudge
from .genai.context_loader import ContextLoader

__version__ = "1.6.7"
__author__ = "Judo Framework Team"

# Main exports
__all__ = [
    "Judo",
    "JudoResponse",
    "Matcher",
    "VariableManager",
    "HttpClient",
    "MockServer",
    "JudoReporter",
    "HTMLReporter",
    "BaseRunner",
    "TestSuite",
    "CommonSuites",
    # GenAI
    "GenAIClient",
    "GenAIJudge",
    "ContextLoader",
]

# Behave integration is available via judo.behave module
# GenAI testing is available via judo.genai module
# Reporting is available via judo.reporting module
# Runners are available via judo.runner module